
%%%   EWKM algorithm   %%%%%%%

function [ACC,ARI,NMI] = EWKM_main(x,Modes)

%%
[n,d]=size(x);
true_class=(x(:,d));
true_class = true_class';
d=d-1;
x=x(:,1:d);
k=max(true_class);
m=1;
for i=1:d
    t=max(x(:,i));
    if t>m
        m=t;
    end
end
Pm.Xlth=n;
Pm.Xwd=d;
Pm.k=k;
Pm.O=4; %*%*%*%*%*%*%*%* set this for different data sets
%%% 0 for pure nominal data, d for pure ordinal data %%%

%%
[x,Pm,AttMtx] = EBDMPre(x,Pm);


%%
value_frequency = zeros(m, d); %%%frequency of each attribute value in the whole data set
for i = 1:n
    for j = 1:d
        value_frequency(x(i, j), j) = value_frequency(x(i, j), j) + 1;
    end
end

miu = Modes;  %%% k modes

f_weight = ones(d, k)./d;  %%% feature weights
gamma = 1.5;


%%
%%%%% Assign the data in the initial k clusters %%%%%%%%%%
cluster_label = zeros(1, n);
cluster_count = zeros(1, k);
for i = 1:n
    [DistVec] = EBDMDist_EWKM(x(i,:),miu,AttMtx,Pm,f_weight);
    [~,j]=min(DistVec);
    cluster_label(i)=j;
    cluster_count(j) = cluster_count(j) + 1;
end
%%
%%%%%%% update the initial k modes %%%%%%%%%%%%%%%%%%%%%%%%%%%
[new_miu, ~] = Update_modes(cluster_label, x, k, m);

%%
%%%%%%%% Iterative steps: adjust the cluster label of each data and update the modes of corresponding clusters
%%%%%%%% untill no data need to change class. %%%%%%%%%%%%%%%%%%%%%%%%%%

moves = 1;
epoch = 0;
while (moves ~= 0)&&(epoch < 50) %%% some class label has been changed in the last step%%%
    moves = 0;
    %%
    %%%%% Update feature weights
    weight_value = zeros(k, d);
    Determin_weight = zeros(k, d);
    for j = 1:d
        for c = 1:k
            for i = 1:n
                if cluster_label(i) == c
                    u_value = 1;
                else
                    u_value = 0;
                end
                distance_value = AttMtx{j}(x(i,j),new_miu(c,j));
                Determin_weight(c, j) = Determin_weight(j) + u_value*distance_value;
            end
            weight_value(c, j) = exp(-Determin_weight(c, j)/gamma);
        end
    end
    weight_value = weight_value';
    weight_regular = sum(weight_value);
    for c = 1:k
        f_weight(:, c) = weight_value(:, c)./weight_regular(c);
    end
    
    %%
    for i = 1:n  %%%update the cluster label of all data
        [DistVec] = EBDMDist_EWKM(x(i,:),new_miu,AttMtx,Pm,f_weight);
        [~,new_label]=min(DistVec);
        if cluster_label(i) ~= new_label  %%% need adjust
            moves = moves + 1;
            old_cluster = cluster_label(i);
            cluster_label(i) = new_label;     %%%%updata cluster label
            cluster_count(new_label) = cluster_count(new_label) + 1;
            cluster_count(old_cluster) = cluster_count(old_cluster) - 1;   %%%%update cluster counter
        end
    end
    [new_miu, ~] = Update_modes(cluster_label, x, k, m);
    epoch = epoch + 1;
end

%%
%%%% Clustering accuracy
[ACC,Confu_Matrix,~,~] = calAccuracy(cluster_label',true_class');

%%
%%% ARI index
ARI = calRI(cluster_label,true_class,'adjusted');

%%
%%% NMI index
[NMI,~,~,~,~] = calNMI(Confu_Matrix, n);

end

            